package com.simonstl.ents;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.IOException;
import java.util.*;
import java.io.PrintStream;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class writes Java code based on the contents of an XML Ents config file.  The resulting class creates a derivative of {@link com.simonstl.ents.EntRules EntRules} which begins with the rules specified by that file.</p>
<p>To use this class from the command-line, run the {@link com.simonstl.ents.EntTester CharTester} class with the -g option.</p>
*
* @version 0.01 4 February 2002
* @author Simon St.Laurent
*/

public class EntRulesGen extends DefaultHandler {
    
    public EntRules rules;   //set of rules
    protected PrintStream output=System.out;
    protected String outPackage="com.simonstl.ents";
    protected String outClass="GeneratedRules";

    // Constructor
    public EntRulesGen (String xmlFile, XMLReader parser) {
        
        //  Set Content Handler
        parser.setContentHandler (this);

        // Create a rules container
        rules=new EntRules();


        //  Parse the Document      
        try {
            parser.parse(xmlFile);
        } catch (SAXException e) {
            System.err.println (e);
        } catch (IOException e) {
            System.err.println (e);
        }
    }

public void setOutput(PrintStream _output) {
    output=_output;
}

public PrintStream getOutput(){
    return output;
}

public void setOutPackage (String _outPackage) {
    outPackage=_outPackage;
}

public String getOutPackage () {
    return outPackage;
}

public void setClass (String _outClass) {
    outClass=_outClass;
}

public String getOutClass () {
    return outClass;
}


public void startDocument() {
//write opening code
output.println("package "+outPackage+";");
output.println("import com.simonstl.ents.*;");
output.println("import java.util.*;");

output.println("/* The contents of this file are subject to the Mozilla Public License  Version 1.1 (the \"License\"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/  \n\nSoftware distributed under the License is distributed on an \"AS IS\"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. \n\nThe Original Code is available at  http://simonstl.com/projects/fragment/original.\n\nThe Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved.\n\n*/");
output.println("/**Generated code created by the com.simonstl.ents.EntRulesGen class.**/");
output.println("public class "+outClass+" extends com.simonstl.ents.EntRules{");
output.println("protected HashMap rulesByEnt=new HashMap();");
output.println("protected HashMap rulesByRef=new HashMap();");
output.println("protected EntRule rule;");

output.println("public "+outClass+" () {");
output.println("  ");
}

public void endDocument() {
//write ending code

output.println("}//end constructor");

output.println("public static void main (String[] args) throws Exception {");
output.println(outClass+" newEnt=new "+outClass+"();");
output.println("newEnt.print();");
output.println("}//end main");

output.println("}//end class");


}


    //  Start Element Event Handler
public void startElement (String uri, String local,
        String qName, Attributes atts)  {

       
        if (uri.equals("http://simonstl.com/ns/refsEnts/")) {
//System.out.println(qName);
           if (local.equals("equal")) {
             String ref=atts.getValue("ref");
             String ent=atts.getValue("ent");

//System.out.println("Ending at "+endInt);
output.println("rule=new EntRule(\""+ent+"\", \""+ref+"\");");
output.println("this.addRule(rule);");
           }


       }//end namespace
}//end startElement

   
}