package com.simonstl.ents;

import com.simonstl.ents.*;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import java.io.IOException;
import java.io.*;
import java.util.*;
import com.megginson.sax.XMLWriter;
import com.megginson.sax.DataWriter;
import java.io.FileReader;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.XMLReaderFactory;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class tests {@link com.simonstl.ents.EntsRefsFilter EntsRefsFilter }.
* It takes two arguments - a rules file and a target file.
* It loads the rules, feeds them to the filter, and displays
*  the results on the command line.</p>
**
* <p>Version 0.01 is based on the FilterTester class from the <a href="http://simonstl.com/projects/fragment">com.simonstl.fragment</a> package.</p>
*
* @version 0.01 4 February 2002
* @author Simon St.Laurent
*/

public class EntTester {
/**
printHelp() lists options if the user doesn't specify any files.
*/

public static void printHelp() {
    System.out.println("Namespace Element FilterTester requires two arguments - a rules file and then a target XML file");
    System.out.println("");
    System.out.println("Options may appear before those arguments:");
    System.out.println("-p [parser class name] - specifies a SAX2 parser for use with this tool");
    System.out.println("-r  - parse and display the rules only");
    System.out.println("-E  - convert char refs to entities");
    System.out.println("More information is available at http://simonstl.com/projects/namespace/");
    System.out.println("");
/*
    System.out.println("");
    System.out.println("");
    System.out.println("");
    System.out.println("");
*/
}

/**
FilterTester's main method requires two arguments - a rules file and a target XML file.  A rules file may also be specified.

Options may appear before those arguments:<br />
-p [parser class name] - specifies a SAX2 parser for use with this tool<br />
-r  - parse and display the rules only<br />

*/

  // Main Method
  public static void main (String[] args) throws Exception {

    int numArgs=args.length;
    boolean printRules=false;
    boolean generateCode=false;

    boolean ents=true;
    String rulesFile;
    String targetFile;

//deal with System properties
   if ((System.getProperty("org.xml.sax.driver"))==null) {
      System.setProperty("org.xml.sax.driver","org.apache.xerces.parsers.SAXParser");
   }


   String parserName=System.getProperty("org.xml.sax.driver");

    if (numArgs<1) {
      printHelp();
    } else {
//process arguments
       rulesFile=args[numArgs-2];
       targetFile=args[numArgs-1];
for (int i=0; i<numArgs; i++) {
   if (args[i].equals ("-p")) {
       parserName=args[i+1];
       i++;
   }
   if (args[i].equals ("-E")) {
       ents=false;
   }
   if (args[i].equals ("-g")) {
       generateCode=true;
       rulesFile=args[i+1];
       i++;
   }
   if (args[i].equals ("-r")) {
       printRules=true;
       rulesFile=args[i+1];
       i++;
   }
}

       //  Create a SAX Parser
      XMLReader rulesParser = XMLReaderFactory.createXMLReader(parserName); 

     if (generateCode) {
        EntRulesGen thisRulesGen = new EntRulesGen (rulesFile, rulesParser);

       System.exit(0);
     }


        EntRulesLoader thisRulesLoader = new EntRulesLoader (rulesFile, rulesParser);
        EntRules rules=thisRulesLoader.rules;


if (printRules) {
  rules.print();
} else {
//  run it
        try {          
            // Create a stream to read and clean the file
            FileReader entReader=new FileReader(targetFile);
            EntsRefsFilter entFilter=new EntsRefsFilter(entReader);
            entFilter.setRules(rules);
            entFilter.setEnts(ents);
            BufferedReader in = new BufferedReader(entFilter);
            String line;
            while((line = in.readLine()) != null) {
              System.out.println(line);
            }
            in.close();  // Close the stream.
        }
        catch(Exception e) {
            e.printStackTrace();
        }

      }
    }
  }
}