/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/o2kxml/. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2000-2002. All Rights Reserved. 

Contributor(s): 
*/

package com.simonstl.ents;


import java.io.*;

/**
 * <p>This class takes character references and converts them into character entities or vice-versa.</p>
 * 
 * 
 * @author Simon St.Laurent
 * @version 0.01 $Date: 2002/02/04 $
 */
public final class EntsRefsFilter extends FilterReader
{
    protected boolean generating=false;
    protected boolean ents=true;
    protected EntRules rules;
    protected StringBuffer myBuffer = new StringBuffer();

    public EntsRefsFilter (InputStream in) {
      this(new InputStreamReader(in));
    }

    public EntsRefsFilter (Reader in) {
      super(new BufferedReader(in));
    }

    public void setRules(EntRules _rules){
      rules=_rules;
    }

    public void setEnts(){
     ents=true;
    }

    public void setEnts(boolean _ents){
     ents=_ents;
    }

    public void setRefs(){
     ents=false;
    }


    protected int feedFromInternalBuffer() {
        //feed out the buffer a character at a time
        int retChar=myBuffer.charAt(0);
        myBuffer.reverse();
        myBuffer.setLength(myBuffer.length()-1);
        myBuffer.reverse();
        return retChar;
    }
    
    public int read() throws IOException {

        int c = ' '; //return a space if nothing else

      if (myBuffer.length()==0) {
       c=in.read();
//System.out.println(c);
        if (c=='&') {
         c=in.read();
         while (!(c==';')){
           myBuffer.append((char)c);
           c=in.read();
         } //finished collecting ref

      
      String testString=myBuffer.toString();
//System.out.println("Testing:"+testString);

      String newVal=null;

      if (ents){
      //check by Ents
      newVal=rules.resultEntRule(testString);
//System.out.println("Result of ent test:"+newVal);
      } else {
      //check by Refs
      newVal=rules.resultRefRule(testString);
//System.out.println("Result of ent test:"+newVal);

      }
      if (newVal==null){
      //ah well.  Just leave it.
      myBuffer.insert(0,'&');
      myBuffer.append(';');
      } else {
      myBuffer=new StringBuffer();
      myBuffer.append('&');
      myBuffer.append(newVal);
      myBuffer.append(';');
      } 

     } //end startEnt (c)
    } //end buffer test


      if (myBuffer.length()>0) {
         c=feedFromInternalBuffer();
      } 

        return c;
    }

    /**
     * This method is a placeholder - all 'real' activity appears in 
     * the int read() method.  
     * This placeholder is substantially from Java I/O by 
     * Elliotte Rusty Harold, http://www.oreilly.com/catalog/javaio/.
     */
    private boolean endOfStream = false;
  
    public int read(char[] text, int offset, int length) throws IOException {
  
        if (endOfStream) return -1;
        int numRead = 0;
        
        for (int i = offset; i < offset+length; i++) {
            int temp = this.read();
            if (temp == -1) {
              this.endOfStream = true;
              break;
            }
            text[i] = (char) temp;
            numRead++;
        }
        return numRead;
  
    }

    /** This class defines a main() method to test the EntRefsFilter.  Doesn't currently work because it needs rules set up. */
    public static void main(String[] args) {
        
        if (args.length != 1) {
            System.out.println("Usage: java EntsRefsFilter <filename>");
            return;       
        }
        try {          
            // Create a stream to read and clean the file
            EntsRefsFilter entFilter=new EntsRefsFilter(new FileReader(args[0]));
            BufferedReader in = new BufferedReader(entFilter);
            String line;
            while((line = in.readLine()) != null) {
              System.out.println(line);
            }
            in.close();  // Close the stream.
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
    
}
